/*
        Based on a Chat program by
	By Dattaraj J. Rao ( dattarajrao@yahoo.com )
*/

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ChatClient extends Frame implements IChatClient,ActionListener
{
	public static void main(String[] args) 
	{
                /*
                 * STUDENTS LOOK HERE
                 *
                 * This is the main method that gets run when this
                 * class is fed to the java VM. All it does is
                 * create an instance of the chat client class
                 * to represent a user
                 */
		ChatClient client=new ChatClient("Chat Client");
	}

	// Class Members //
	IChatService service=null; // The server's proxy.

	TextField servername,serverport,username;
	Button connect,disconnect;

	TextField message;
	Button send;
	TextArea fromserver;

	boolean connected=false;
	String title,user="";

	public ChatClient()
	{
	}

	public ChatClient(String str)
	{
		super(str);

                // This isn't really necessary, but I left it in.
		System.setSecurityManager(new RMISecurityManager());

                
                /*
                 * This is just building the GUI
                 */
		setBounds(50,20,400,400);
		setLayout(new FlowLayout(FlowLayout.CENTER,25,10));

		title=str;
		setStatus();

		// Create controls //
		add(new Label("Chat Server Name : "));
		servername=new TextField(20);
		add(servername);
		servername.setText("localhost");

		add(new Label("Chat Server Port   :  "));
		serverport=new TextField(20);
		add(serverport);
		serverport.setText("1055");

		add(new Label("Your User Name : "));
		username=new TextField(20);
		add(username);
		username.setText("raj");

		connect=new Button("Connect");
		connect.addActionListener(this);
		add(connect);

		disconnect=new Button("Disconnect");
		disconnect.addActionListener(this);
		add(disconnect);

		message=new TextField(30);
		add(message);

		send=new Button("Send");
		send.addActionListener(this);
		add(send);

		fromserver=new TextArea(10,50);
		add(fromserver);
		fromserver.setEditable(false);

              
                
                /*
                 * STUDENTS LOOK HERE
                 *
                 * Okay, the client is all set up and ready to go. 
                 * But note that it isn't dervied from Remote, 
                 * for example, UnicastRemoteObject.
                 *
                 * This is a problem, because the server needs to
                 * write unsolicited messages to it.
                 *
                 * This is because Java doesn't support mutliple
                 * inheritence, and it is already inheriting from
                 * Frame to build the interface. 
                 * 
                 * So, we need to explicitly ask to have this object
                 * exported, something which is normally done in the
                 * constructor of a remote object.
                 *
                 * Take a look in the API documentation at the 
                 * UnicastRemoteObject class. It'll have a static 
                 * method that can help here.
                 *
                 * Remember, you want to _export_ _this_ _object_.
                 */

		setVisible(true);
	}

	// RMI connection //
	private void connect()
	{
		try
		{
			// Get port //
			int port;
			try
			{
				port=Integer.parseInt(serverport.getText());
			}
			catch(Exception e)
			{
				port=1055;
			}

                        /*
                         * Here you need to locate the registry running on
                         * "servername" at port "port". 
                         *
                         * Then, take the resulting reference and use it
                         * to lookup the reference to the "ChatService"
                         * object. This reference should be stored in the
                         * "service" reference. 
                         */




			service.addClient(this);
			connected=true;
			setStatus();
			user=username.getText();
		}
		catch(Exception e)
		{
			fromserver.append("Error Connecting ...\n");
			connected=false;
			setStatus();
			service=null;
			System.out.println(e);
		}
	}

	private void disconnect()
	{
		try
		{
			if(service==null)
				return;
			
			service.removeClient(this);
			service=null;
		}
		catch(Exception e)
		{
			fromserver.append("Error Connecting ...\n");
		}
		finally
		{
			connected=false;
			setStatus();
		}
	}

	private void setStatus()
	{
		if(connected)
			 setTitle(title+" : Connected");
		else
			 setTitle(title+" : Not Connected");
	}

	// IChatClient methods //

	public String getName()
	{
		return user;
	}

	public void sendMessage(String msg)
	{
		fromserver.append(msg+"\n");
	}

	// Actionlistener //

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource()==connect)
		{
			connect();

			if(connected)
			{
				servername.setEnabled(false);
				serverport.setEnabled(false);
				username.setEnabled(false);
				connect.setEnabled(false);
			}
		}
		else
		if(e.getSource()==disconnect)
		{
			disconnect();

			servername.setEnabled(true);
			serverport.setEnabled(true);
			username.setEnabled(true);
			connect.setEnabled(true);
		}
		else
		if(e.getSource()==send)
		{
			if(service==null)
				return;
			
			try
			{
				service.sendMessage(this,message.getText());
				message.setText("");
			}
			catch(RemoteException exp)
			{
				System.out.println(exp);
				fromserver.append("Error Sending Message ...\n");
			}
		}
	}
}
